package org.w3c.rdf.util;

import org.w3c.rdf.model.*;
import java.util.*;

/**
 * An implementation of set operations on models.
 * Every of these static methods first attempts to use the native implementation provided by the model if any.
 */

public class SetOperations {

  public static RDFModel union(RDFModel m1, RDFModel m2) {

    RDFModel res = m1.duplicate();
    unite(res, m2);
    return res;
  }

  public static void unite(RDFModel m1, RDFModel m2) {

    if(m1 instanceof SetModel)
      ((SetModel)m1).unite(m2);
    else {
      synchronized(m2) {

	for(Enumeration en = m2.elements(); en.hasMoreElements();)
	  m1.add( (Triple)en.nextElement() );
      }
    }
  }

  public static RDFModel difference(RDFModel m1, RDFModel m2) {

    RDFModel res = m1.duplicate();
    subtract(res, m2);
    return res;
  }

  public static void subtract(RDFModel m1, RDFModel m2) {

    if(m1 instanceof SetModel)
      ((SetModel)m1).subtract(m2);
    else {
      synchronized(m2) {

	for(Enumeration en = m2.elements(); en.hasMoreElements();)
	  m1.remove( (Triple)en.nextElement() );
      }
    }
  }

  public static RDFModel intersection(RDFModel m1, RDFModel m2) {

    RDFModel res = m1.duplicate();
    intersect(res, m2);
    return res;
  }

  public static void intersect(RDFModel m1, RDFModel m2) {

    if(m1 instanceof SetModel)
      ((SetModel)m1).intersect(m2);
    else {
      synchronized(m2) {

	RDFModel tmp = m1.duplicate();

	for(Enumeration en = tmp.elements(); en.hasMoreElements();) {
	  Triple t = (Triple)en.nextElement();
	  if(!m2.contains(t))
	    m1.remove(t);
	}
      }
    }
  }

}
